# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
# IDs of services provided by this plugin
IPROJECT_MODEL = 'envisage.ui.single_project.model_service.ModelService'
IPROJECT_UI = 'envisage.ui.single_project.ui_service.UiService'


#### EOF #####################################################################
